#!/usr/bin/php
<?
//Assembler for the Nabaztag VM, written in PHP.
//What it does: take your instructions, spit out a binary.
//It can do simple calculations, like 'ld r1,'A'+3' too. No brackets, plz.
//What it doesn't do: macro processing (please use cpp or something
//alike if you want that), emit complete Type-5 frames.

//This code is licensed under the gnu GPL. Please refer to
//http://www.gnu.org/licenses/gpl.html for more info.

//Copyright 2006-2007, Jeroen Domburg.




//The instruction type, as defined in the Violet vm asm manual, paragraph 4.8
//0=(inherent), 1=I, 2=R, 3=R,I, etc
//instype[first_byte_of_opcode] gives the type of that opcode.


$instype=array(
//  0 1 2 3 4 5 6 7 8 9 A B C D E F
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //LD
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //ADD
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //SUB
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //AND
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //OR
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //LDR
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, //STR
    0,0,0,0,0,4,4,4,4,4,4,2,2,2,1,2, //7
    2,2,2,2,2,2,4,4,4,4,4,4,4,4,4,4, //8
    4,4,4,4,4,6,4,6,9,8,8,7,7,7,7,7, //9
    7,7,7,7,7,7,7,4,4,1,5,5,7,0,4,2, //A
    0,2,0,4,4,2,2,2,2,4,8,8,4,4,4,0, //B
    4,2,2,0,4,2,4,0,0,0,0,0,0,0,0,0, //C
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, //D
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, //E
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  //F
	);

//mnemonics for instrs 0-0x6F
$mnem_h=array("LD","ADD","SUB","AND","OR","LDR","STR");

//mnemonics for 70-end
$mnem_l=array(
    "NOP","","RTI","CLRCC","SETCC","ADDCC","SUBCC","INCW",
    "DECW","MULW","INPUTRST","INT","","WAIT","WAIT","RND",
    "DEC","INC","CLR","NEG","NOT","TST","LD","ADD",
    "SUB","MUL","AND","OR","EOR","LSL","LSR","ASR",
    "ROL","ROR","CMP","BIT","LDR","LDR","STR","STR",
    "LDT","LDTW","INPUT","RTIJ","BRA","BEQ","BNE","BGT",
    "BGE","BLT","BLE","BHI","BHS","BLO","BLS","LED",
    "PALETTE","","PUSH","PULL","BSR","RTS","MOTOR","MIDIPLAY",
    "MIDISTOP","WAVPLAY","WAVSTOP","MSEC","SEC","BUT3","VOL","MVOL",
    "PUSHBUTTON","SRC","BRAT","BSRT","OSC","INV","DIV","HSV",
    "MOTORGET","MUSIC","DOWNLOAD","","SEND","SENDREADY","LASTPING");

if ($argc!=3) {
    echo "Usage: ".$argv[0]." input.nzasm output.bin\n";
}


function parseme($word,$pass) {
    global $labels;
    $word=trim($word);
//    echo "parsing $word\n";
    $p=strrpos($word,"-");
    if ($p!==FALSE) {
	if (p==0) 
	 return -parseme(substr($word,0,$p));
	else
	 return parseme(substr($word,$p+1))-parseme(substr($word,0,$p));
    }
    $p=strrpos($word,"+");
    if ($p!==FALSE) {
	return parseme(substr($word,$p+1))+parseme(substr($word,0,$p));
    }
    $p=strrpos($word,"/");
    if ($p!==FALSE) {
	return parseme(substr($word,$p+1))/parseme(substr($word,0,$p));
    }
    $p=strrpos($word,"*");
    if ($p!==FALSE) {
	return parseme(substr($word,$p+1))*parseme(substr($word,0,$p));
    }
    
    if (preg_match("/^[0-9.]+$/",$word)) return floatval($word);
    if (preg_match("/^\\$[0-9ABCDEFabcdef.]+$/",$word)) return hexdec(substr($word,1));
    if (preg_match("/^0x[0-9ABCDEFabcdef.]+$/",$word)) return hexdec(substr($word,2));
    if (preg_match("/^'.'$/",$word)) return ord(substr($word,1));
    if ($labels[$word]!=0) return $labels[$word];
    if ($pass==2) {
	echo "?Parser didn't understood '$word'\n";
	return FALSE;
    } else {
	return -1;
    }
}

function whatsthis($word,&$val,$pass) {
    global $labels;
//    echo "Whatis $word ?\n";
    if ($word=="") return("-");
    if (preg_match("/^[a-zA-Z0-9_-]+:$/",$word)) return("l");
    if (preg_match("/^[rR][0-9]+$/",$word)) {
	$val=substr($word,1);
	return("r");
    }
    //No reg, no I; must be an expression. Parse it.
    $val=parseme($word,$pass);
    if ($val===FALSE) return "e";
    return "i";
}


function lookup_opcode($ins,$type) {
    //Watch out: type=3 could mean type=8 just as well.
    global $instype;
    global $mnem_h;
    global $mnem_l;
    $op=-1;
    if ($type==2 || $type==3) {
	reset($mnem_h);
	while (list($possop,$val)=each($mnem_h)) {
	    if (strtoupper($ins)==$val) $op=$possop<<4;
	}
    }
    if ($op==-1) {
	reset($mnem_l);
	while (list($possop,$val)=each($mnem_l)) {
	    $possop+=0x70;
//	    if (strtoupper($ins)==$val) echo "Possible: ".dechex($possop)."\n";
	    if (strtoupper($ins)==$val && $instype[$possop]==$type) $op=$possop;
	    //cause we can't be sure from the asm listing if a word or byte val is meant.
	    if (strtoupper($ins)==$val && $instype[$possop]==8 && $type==3) $op=$possop;
	    if (strtoupper($ins)==$val && $instype[$possop]==7 && $type==1) $op=$possop;
	}
    }
    if ($op==-1) {
	echo("Sorry, I don't know any instruction called $ins with opcode types $type.");
	return -1;
    }
    echo "Ins=$ins, op=".dechex($op).", type=".$instype[$op]."\n";
    return $op;
}

function assemble($line,$pc,$pass) {
    global $instype;
    global $labels;
    if ($line=="") return;
//    echo "Line: $line\n";
    if (preg_match('/^[a-zA-Z0-9_-]+:/',$line)) {
	$label=substr($line,0,strpos($line,":"));
	$labels[$label]=$pc;
//	echo "Label: $label @ $pc\n";
	$line=trim(substr($line,strpos($line,":")+1));
    }
    if ($line=="") return;
    $line.=" ";
    $p=strpos($line," ");
    $ins=substr($line,0,$p);
    $line=trim(substr($line,$p+1));
//    echo "Opcode: $ins\n";
    $otypes="";
    for ($i=0; $i<3; $i++) {
	$p=strpos($line,",");
	if ($p!==FALSE) {
	    $operand=substr($line,0,$p);
	    $otypes.=whatsthis($operand,$v,1);
	    $val[$i]=$v;
	    $line=trim(substr($line,$p+1));
	} else {
	    $otypes.=whatsthis($line,$v,1);
	    $val[$i]=$v;
	    $otypes=substr($otypes."---",0,3);
	    break;
	}
    }
    
//    echo "Total: ".$ins."_".$otypes."\n";
    $type=-1;
    if ($otypes=="---") $type=0;
    if ($otypes=="i--") $type=1;
    if ($otypes=="r--") $type=2;
    if ($otypes=="ri-") $type=3;
    if ($otypes=="rr-") $type=4;
    if ($otypes=="ii-") $type=5;
    if ($otypes=="rir") $type=6;
    //Type 7 == type 1, as far as we can see in the asm listing. Decide that later.
    //Type 8 == type 3, as far as we can see in the asm listing. Decide that later.
    if ($otypes=="rri") $type=9;
    if ($type==-1) {
	echo "Invalid operand mode: $otypes at the following line:\n$line\n";
	exit(0);
    }
    $opcode=lookup_opcode($ins,$type);
    if ($opcode==-1) {
	echo "Invalid opcode: $ins $otypes at the following line:\n$line\n";
	exit(0);
    }
    $ret[1]=$opcode;
    if ($type==0) {
	$ret[0]=1;
    } else if ($type==1) {
	if ($instype[$ret[1]]==1) {
	    $ret[2]=$val[0];
	    $ret[0]=2;
	} else { //type 7 disguised as a type 1
	    $ret[2]=($val[0]>>8);
	    $ret[3]=($val[0]&0xff);
	    $ret[0]=3;
	}
    } else if ($type==2) {
	$ret[2]=$val[0];
	$ret[0]=2;
    } else if ($type==3) {
	if ($instype[$ret[1]]==3) {
	    $ret[1]|=$val[0];
	    $ret[2]=$val[1];
	    $ret[0]=2;
	} else { //was a type 8 disguised as a type 3
	    $ret[2]=$val[0];
	    $ret[3]=($val[1]>>8);
	    $ret[4]=($val[1]&0xff);
	    $ret[0]=4;
	}
    } else if ($type==4) {
	$ret[2]=($val[0]<<4)|$val[1];
	$ret[0]=2;
    } else if ($type==5) {
	$ret[2]=$val[0];
	$ret[3]=$val[1];
	$ret[0]=3;
    } else if ($type==6) {
	$ret[2]=($val[0]<<4)|$val[2];
	$ret[3]=$val[1];
	$ret[0]=3;
    } else if ($type==9) {
	$ret[2]=($val[0]<<4)|$val[1];
	$ret[3]=($val[2]>>8);
	$ret[4]=($val[2]&0xFF);
	$ret[0]=1;
    }
    return $ret;
}

//First pass. Labels etc are unknown yet, so we can't spit out the binary.
echo "Pass 1...\n";
$infile=fopen($argv[1],"r");
$pc=17;
while ($in=fgets($infile,1024)) {
    //strip comments and empty lines
    $in=preg_replace("/[#;].*$/","",$in);
    $res=assemble(trim($in),$pc,1);
    $pc+=$res[0];
}
fclose($infile);

//Second pass. Labels should be known by now.
echo "Pass 2...\n";
$infile=fopen($argv[1],"r");
$pc=17;
$out="";
while ($in=fgets($infile,1024)) {
    //strip comments and empty lines
    $in=preg_replace("/[#;].*$/","",$in);
    $res=assemble(trim($in),$pc,2);
    $pc+=$res[0];
    for ($i=0; $i<$res[0]; $i++) {
	$out.=chr($res[1+$i]);
	printf("%02X ",$res[1+$i]);
    }
    echo "    ".trim($in)."\n";
}
fclose($infile);
echo "Done.\n";
$outfile=fopen($argv[2],"w");
fwrite($outfile,$out);
fclose($outfile);
?>