//Routines to update the virtual nabaztags VM firmware image and data from
//an HTTP-server
#include <curl/curl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "machine.h"
#include "config.h"

#define RESPBUFFSIZE 102400 //Buffer-size, in bytes, of the http response

CURL *easyhandle;
unsigned char respbuff[RESPBUFFSIZE];
long respbuffpos;
size_t update_getdata(void *ptr, size_t size, size_t nmemb, void *stream);
int nabaz_status;
long lastid=0;
long prevlastid=0;

//Init update system
int update_init() {
    if (curl_global_init(CURL_GLOBAL_NOTHING)!=0) return(1);
    easyhandle=curl_easy_init();
    if (easyhandle==NULL) return(1);
    nabaz_status=0;
    return(0);
}

//Get a long word from a message in an array of bytes.
long getlong(unsigned char *msg) {
    return ((long)msg[0]<<24)+(msg[1]<<16)+(msg[2]<<8)+msg[3];
}

//Handle type 5 message (firmware image + music/sound)
void msghandledown(unsigned char* msg, long size) {
    long id;
    int trans,f;
    long musptr,musptrnext,mussize;
    long nomus, n;
    char buff[30];

    //Debug: Write msg out to file
    f=open("fw.bin",O_WRONLY|O_CREAT|O_TRUNC,0644);
    write(f,msg,size);
    close(f);
    
    if (strncmp((char*)msg,"amber",5)!=0) {
	printf("No magic signature found!\n");
	return;
    }
    id=getlong(msg+5);
    prevlastid=lastid;
    lastid=id;
    trans=msg[9];
    size=getlong(msg+10);
    printf("Got a new firmware image, id=%04lX size=%li.\n",id,size);
    memcpy(mach_progmem+17,msg+14,size);


    nomus=getlong(msg+14+size);
    printf("Firmware incorporates %li midi tracks / pcm files.\n",nomus);
    for (n=0; n<nomus; n++) {
	musptr=getlong(msg+size+18+((n-1)*4))+(size+18+(nomus*4));
	if (n==0) musptr=(size+18+(nomus*4));
	printf("Curr ptr = %lx\n",musptr);
	musptrnext=getlong(msg+size+18+((n)*4))+(size+18+(nomus*4));
	printf("Next ptr = %lx\n",musptrnext);
	mussize=musptrnext-musptr;
	sprintf(buff,"storage/%04li",n);
	f=open(buff,O_WRONLY|O_CREAT|O_TRUNC,0644);
	write(f,msg+musptr,mussize);
	close(f);
	printf("Wrote %li octets to %s.\n",mussize,buff);
    }
//This code segfaults with 0 midi/audio tracks.
//    if (strncmp((char*)msg+musptr,"mind",4)!=0) {
//	printf("mind-magic not found!\n");
//    }
    if (trans==1) { //direct transition requested? Reset machine
	printf("New fw downloaded and server requests reset.\n");
	mach_vm_reset();
    }
}

void msghandleparm(unsigned char* msg, long size) {
    long id;
    id=getlong(msg);
    printf("Handle ram parameters, id=%lx\n",id);
    memcpy(mach_sources,msg+4,size);
}


int loadbinfile(char *filename) {
    int f,n;
    f=open(filename,O_RDONLY);
    n=read(f,respbuff,RESPBUFFSIZE);
    if (n<=0) return(1);
    close(f);
    printf("File %s loaded, %i bytes.\n",filename, n);
    msghandledown(respbuff+4, n-4);
    return(0);
}


void msghandledelay(unsigned char* msg, long size) {
    int i;
    i=msg[0];
    printf("Unimplemented: setting delay to %i\n",i);
}



int send_do(long sd) {
    char url[2048];
    int f;
    long mptr;
    
    if (sd != -1) {
	snprintf(url, 2048,
	    "http://%s/vl/FR/p3.jsp?v=12&sn=%s&st=%02i&ts=%08lX&sd=%04lX&tc=%08lX&tn=%08lX",
	    SERVER, MYMAC, nabaz_status, lastid, sd, lastid, prevlastid);
    } else {
	snprintf(url, 2048,
	    "http://%s/vl/FR/p3.jsp?v=12&sn=%s&st=%02i&tc=%08lX&tn=%08lX",
	    SERVER, MYMAC, nabaz_status, lastid, prevlastid);
    }
    curl_easy_reset(easyhandle);
    curl_easy_setopt(easyhandle, CURLOPT_URL, url);
    respbuffpos=0;
    curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, update_getdata);
    printf("Getting %s...\n",url);
    curl_easy_perform(easyhandle);
    printf("Done. Got %li bytes.\n",respbuffpos);
    nabaz_status=1;

    //Debug: Write msg out to file
    f=open("packet.bin",O_WRONLY|O_CREAT|O_TRUNC,0644);
    write(f,respbuff,respbuffpos);
    close(f);
    
    //Interpret messages
    if (respbuff[0]!=0x7f) {
	printf("Packet header != 0x7f! Bailing out.\n");
	return(1);
    }

    //Handle all the messages in the packet    
    mptr=1;
    while (mptr<=respbuffpos && respbuff[mptr]!=0xff) {
	int mtype;
	long msize;
	mtype=respbuff[mptr];
	msize=((long)respbuff[mptr+1]<<16)+(respbuff[mptr+2]<<8)+respbuff[mptr+3];
	printf("Found message of type %i, length %li.\n",mtype,msize);
	if (mtype==3) {
	    msghandledelay(respbuff+mptr+4,msize);
	} else if (mtype==5) {
	    msghandledown(respbuff+mptr+4,msize);
	} else if (mtype==4) {
	    msghandleparm(respbuff+mptr+4,msize);
	} else {
	    printf("Msg type not recognized!\n");
	}
	mptr+=msize+4;
    }
    if (respbuff[mptr]==0xff) printf("All messages done.\n");
    return(0);    
}

int update_do() {
    return send_do(-1);
}


size_t update_getdata(void *ptr, size_t size, size_t nmemb, void *stream) {
//    printf("Reading %i bytes from site...\n",size*nmemb);
    if (size*nmemb+respbuffpos>RESPBUFFSIZE) return(0);
    memcpy(respbuff+respbuffpos,ptr,size*nmemb);
    respbuffpos+=size*nmemb;
    return(size*nmemb);
}

